@@ -21,14 +21,17 @@ from os import makedirs, path
 from cmd import Cmd
 from subprocess import Popen, PIPE
 
+from config import Config
+
 STDOUT, STDERR = range(2)
+config = Config()
+
 class Aptly(Cmd):
     def __init__(self):
         Cmd.__init__(self)
         self.prompt = '<aptly> '
-        self.intro = ("Aptly 2.0a1\n"
+        self.intro = ("Aptly 2010.5.15\n"
                       "Type 'help' for more information or 'quit' to exit.")
-        self.options = []
         self.known_commands = ['help', 'install', 'update', 'remove', 'prompt', 'set', 'quit']
         self.commands = []
 
@@ -104,8 +107,10 @@ class Aptly(Cmd):
 
                 Popen(['dpkg', '-i', deb]).communicate()
 
-            Popen(' '.join(['apt-get', 'install'] + self.options + remote),
-                   shell=True).communicate()
+            Popen(' '.join(['apt-get', 'install'] + config.main['options'] + remote),
+                  shell=True).communicate()
+
+
         else:
             packages = raw_input('What package(s) would you like to install?\n'
                                  '===> ')
@@ -114,15 +119,15 @@ class Aptly(Cmd):
 
     def help_install(self):
         print ("install [package1 package2 package3]\n"
-               "  Installs the named packages. If packages are omitted, aptly "
+               "\tInstalls the named packages. If packages are omitted, aptly "
                "will prompt you for package names. \n"
-               "  You may exit this prompt by using the package name 'cancel'.")
+               "\tYou may exit this prompt by using the package name 'cancel'.")
 
     def do_remove(self, packages):
         packages = packages.replace(',', ' ').split()
 
         if packages:
-            Popen(['apt-get', 'remove'] + packages).communicate()
+            Popen(['apt-get', 'remove'] + packages, stdout=PIPE).communicate()
         else:
             packages = raw_input('What package(s) would you like to remove?\n'
                                  '===> ')
@@ -131,20 +136,32 @@ class Aptly(Cmd):
 
     def help_remove(self):
         print ("remove [package1 package2 package3]\n"
-               "  Removes the named packages. If packages are omitted, aptly "
+               "\tRemoves the named packages. If packages are omitted, aptly "
                "will prompt you for package names. \n"
-               "  You may exit this prompt by using the package name 'cancel'.")
+               "\tYou may exit this prompt by using the package name 'cancel'.")
 
     def do_update(self, line):
         Popen(['apt-get', 'update']).communicate()
 
     def do_upgrade(self, line):
-        # TODO: ask the user if they would like to update first
+        if config.main['ask-first']:
+            config.main['update-first'] = raw_input('Would you like to update '
+                                                    'first? (Y/n)\n'
+                                                    '===> ')
+
+        if config.main['update-first']:
+            self.do_update(None)
+
         Popen(['apt-get', 'upgrade']).communicate()
 
-    def do_list(self, package):
-        # TODO: dpkg -L <packages> : list all files installed by packages
-        pass
+    def do_list(self, packages):
+        for package in packages.split():
+            print "Files installed by '%s': " % package
+            Popen(['dpkg', '-L', package]).communicate()
+
+    def help_list(self):
+        print ("list [package1 package2]\n"
+               "\tLists all files installed by named packages.")
 
     def do_set(self, options):
         def get_cache_directory():
@@ -167,7 +184,7 @@ class Aptly(Cmd):
             if key == 'cache':
                 #TODO: see about moving this outside of this enclosure
                 if '-o' not in self.options:
-                    self.options.insert(0, '-o')
+                    config.main['options'].insert(0, '-o')
 
                 if options[key] == '':
                     options[key] = get_cache_directory()
@@ -182,7 +199,7 @@ class Aptly(Cmd):
                     if errno == 17:
                         pass
                     else:
-                        print '%s could not created.' % options[key]
+                        print '%s could not be created.' % options[key]
                         options[key] = get_cache_directory()
                         if not options[key]:
                             break
@@ -193,11 +210,17 @@ class Aptly(Cmd):
                 except OSError:
                     pass
 
-                self.options.append("dir::cache::archives='%s'" % options[key])
+                config.main['options'].append("dir::cache::archives='%s'" % options[key])
+            elif key == 'ask-first':
+                self.config.main[key] = options[key]
+                
     
     def help_set(self):
         print ("set miscellaneous options. Currently available options:\n"
-               "\tcache <directory>")
+               "\tcache <directory> - directory to download deb files\n"
+               "\task-first (Yes/No - whether or not to ask about certain "
+               "actions. For example, if No, you will not be asked whether "
+               "or not you want to update before upgrading.")
 
     def do_prompt(self, new_prompt):
         if new_prompt:
@@ -209,12 +232,15 @@ class Aptly(Cmd):
 
     def help_prompt(self):
         print ("prompt new_prompt\n"
-               "  Change the default prompt (<aplty>) to new_prompt\n"
-               "  If new_prompt is not given, the default prompt is used.")
+               "\tChange the default prompt (<aplty>) to new_prompt\n"
+               "\tIf new_prompt is not given, the default prompt is used.")
 
     def do_quit(self, line):
         sys.exit(1)
 
+    def do_exit(self, line):
+        self.do_quit(None)
+
 
 if __name__ == '__main__':
     if len(sys.argv) > 1:
